package ru.sweetbread.unn.db

import androidx.room.ColumnInfo
import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Entity
import androidx.room.Insert
import androidx.room.PrimaryKey
import androidx.room.Query
import ru.sweetbread.unn.Auditorium
import ru.sweetbread.unn.Building
import ru.sweetbread.unn.Discipline
import ru.sweetbread.unn.KindOfWork
import ru.sweetbread.unn.Lecturer
import ru.sweetbread.unn.LecturerRank
import ru.sweetbread.unn.ScheduleUnit
import ru.sweetbread.unn.ui.layout.db
import java.time.LocalDate
import java.time.LocalTime
import java.time.format.DateTimeFormatter

@Entity
data class BuildingDB(
    @PrimaryKey val oid: Int,
    @ColumnInfo val name: String,
    @ColumnInfo val gid: Int
)

@Dao
interface BuildingDao {
    @Query("SELECT * FROM buildingdb WHERE oid = :oid LIMIT 1")
    fun get(oid: Int): BuildingDB

    @Insert
    fun insert(building: BuildingDB)

    @Delete
    fun delete(building: BuildingDB)
}

fun cacheBuilding(building: Building) {
    try {
        db.buildingDao().insert(
            BuildingDB(
                building.oid,
                building.name,
                building.gid
            )
        )
    } catch (_: android.database.sqlite.SQLiteConstraintException) {
    }
}

fun loadBuilding(oid: Int): Building {
    return db.buildingDao().get(oid).let {
        Building(
            it.name,
            it.gid,
            it.oid
        )
    }
}

@Entity
data class AuditoriumDB(
    @PrimaryKey val oid: Int,
    @ColumnInfo val name: String,
    @ColumnInfo val floor: Int?,
    @ColumnInfo val buildingOid: Int
)

@Dao
interface AuditoriumDao {
    @Query("SELECT * FROM auditoriumdb WHERE oid = :oid LIMIT 1")
    fun get(oid: Int): AuditoriumDB

    @Insert
    fun insert(auditorium: AuditoriumDB)

    @Delete
    fun delete(auditorium: AuditoriumDB)
}

fun cacheAuditorium(auditorium: Auditorium) {
    try {
        cacheBuilding(auditorium.building)
        db.auditoriumDao().insert(
            AuditoriumDB(
                auditorium.oid,
                auditorium.name,
                auditorium.floor,
                auditorium.building.oid
            )
        )
    } catch (_: android.database.sqlite.SQLiteConstraintException) {
    }
}

fun loadAuditorium(oid: Int): Auditorium {
    return db.auditoriumDao().get(oid).let {
        Auditorium(
            it.name,
            it.oid,
            it.floor ?: 0,
            loadBuilding(it.buildingOid)
        )
    }
}

@Entity
data class DisciplineDB(
    @PrimaryKey val oid: Int,
    @ColumnInfo val name: String,
    @ColumnInfo val type: Int
)

@Dao
interface DisciplineDao {
    @Query("SELECT * FROM disciplinedb WHERE oid = :oid LIMIT 1")
    fun get(oid: Int): DisciplineDB

    @Insert
    fun insert(discipline: DisciplineDB)

    @Delete
    fun delete(discipline: DisciplineDB)
}

fun cacheDiscipline(discipline: Discipline) {
    try {
        db.disciplineDao().insert(
            DisciplineDB(
                discipline.oid,
                discipline.name,
                discipline.type
            )
        )
    } catch (_: android.database.sqlite.SQLiteConstraintException) {
    }
}

fun loadDiscipline(oid: Int): Discipline {
    return db.disciplineDao().get(oid).let {
        Discipline(
            it.name,
            it.oid,
            it.type
        )
    }
}

@Entity
data class KindOfWorkDB(
    @PrimaryKey val oid: Int,
    @ColumnInfo val name: String,
    @ColumnInfo val uid: String,
    @ColumnInfo val complexity: Int
)

@Dao
interface KindOfWorkDao {
    @Query("SELECT * FROM kindofworkdb WHERE oid = :oid LIMIT 1")
    fun get(oid: Int): KindOfWorkDB

    @Insert
    fun insert(kindOfWork: KindOfWorkDB)

    @Delete
    fun delete(kindOfWork: KindOfWorkDB)
}

fun cacheKindOfWork(kindOfWork: KindOfWork) {
    try {
        db.kindOfWorkDao().insert(
            KindOfWorkDB(
                kindOfWork.oid,
                kindOfWork.name,
                kindOfWork.uid,
                kindOfWork.complexity
            )
        )
    } catch (_: android.database.sqlite.SQLiteConstraintException) {
    }
}

fun loadKindOfWork(oid: Int): KindOfWork {
    return db.kindOfWorkDao().get(oid).let {
        KindOfWork(
            it.name,
            it.oid,
            it.uid,
            it.complexity
        )
    }
}

@Entity
data class LecturerDB(
    @PrimaryKey val unnId: Int,
    @ColumnInfo val name: String,
    @ColumnInfo val rank: LecturerRank,
    @ColumnInfo val email: String,
    @ColumnInfo val uid: String
)

@Dao
interface LecturerDao {
    @Query("SELECT * FROM lecturerdb WHERE unnId = :unnId LIMIT 1")
    fun get(unnId: Int): LecturerDB

    @Insert
    fun insert(lecturer: LecturerDB)

    @Delete
    fun delete(lecturer: LecturerDB)
}

fun cacheLecturer(lecturer: Lecturer) {
    try {
        db.lecturerDao().insert(
            LecturerDB(
                lecturer.unnId,
                lecturer.name,
                lecturer.rank,
                lecturer.email,
                lecturer.uid
            )
        )
    } catch (_: android.database.sqlite.SQLiteConstraintException) {
    }
}

fun loadLecturer(unnId: Int): Lecturer {
    return db.lecturerDao().get(unnId).let {
        Lecturer(
            it.name,
            it.rank,
            it.email,
            it.unnId,
            it.uid
        )
    }
}

@Entity
data class ScheduleUnitDB(
    @PrimaryKey val oid: Int,
    @ColumnInfo val date: String,
    @ColumnInfo val stream: String,
    @ColumnInfo val begin: String,
    @ColumnInfo val end: String,

    @ColumnInfo val auditoriumOid: Int,
    @ColumnInfo val disciplineOid: Int,
    @ColumnInfo val kindOfWorkOid: Int,
    @ColumnInfo val lecturerId: Int  // TODO: many-to-many
)
@Dao
interface ScheduleItemDao {
    @Query("SELECT * FROM scheduleUnitDB WHERE oid = :oid LIMIT 1")
    fun getScheduleItem(oid: Int): ScheduleUnitDB?

    @Query("SELECT * FROM scheduleUnitDB WHERE date = :date ORDER BY `begin`")
    fun getSchedule(date: String): List<ScheduleUnitDB>

    @Insert
    fun insert(user: ScheduleUnitDB)

    @Delete
    fun delete(user: ScheduleUnitDB)
}

fun cacheScheduleItems(items: ArrayList<ScheduleUnit>) {
    for (item in items) {
        try {
            cacheAuditorium(item.auditorium)
            cacheDiscipline(item.discipline)
            cacheKindOfWork(item.kindOfWork)
            cacheLecturer(item.lecturers[0])

            db.scheduleDao().insert(
                ScheduleUnitDB(
                    item.oid,
                    item.date.format(DateTimeFormatter.ISO_DATE),
                    item.stream,
                    item.begin.format(DateTimeFormatter.ISO_TIME),
                    item.end.format(DateTimeFormatter.ISO_TIME),
                    item.auditorium.oid,
                    item.discipline.oid,
                    item.kindOfWork.oid,
                    item.lecturers[0].unnId
                )
            )
        } catch (_: android.database.sqlite.SQLiteConstraintException) {
        }
    }
}

fun loadSchedule(date: LocalDate): ArrayList<ScheduleUnit> {
    return ArrayList(db.scheduleDao().getSchedule(date.format(DateTimeFormatter.ISO_DATE)).map {
        ScheduleUnit(
            it.oid,
            loadAuditorium(it.auditoriumOid),
            LocalDate.parse(it.date, DateTimeFormatter.ISO_DATE),
            loadDiscipline(it.disciplineOid),
            loadKindOfWork(it.kindOfWorkOid),
            arrayListOf(loadLecturer(it.lecturerId)),
            it.stream,
            LocalTime.parse(it.begin, DateTimeFormatter.ISO_TIME),
            LocalTime.parse(it.end, DateTimeFormatter.ISO_TIME)
        )
    })

}